package tech.mercantec.easyeat.ui.shopping_list

import android.app.AlertDialog
import android.app.Dialog
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ArrayAdapter
import android.widget.EditText
import android.widget.Spinner
import android.widget.Toast
import androidx.fragment.app.Fragment
import tech.mercantec.easyeat.R
import tech.mercantec.easyeat.databinding.FragmentShoppingListBinding
import tech.mercantec.easyeat.helpers.ApiRequestException
import tech.mercantec.easyeat.helpers.addShoppingItem
import tech.mercantec.easyeat.helpers.getShoppingList
import tech.mercantec.easyeat.helpers.toggleShoppingItemChecked
import tech.mercantec.easyeat.models.Dish
import tech.mercantec.easyeat.models.ShoppingListItem
import java.util.ArrayList
import kotlin.concurrent.thread

class ShoppingListFragment : Fragment() {

    private var _binding: FragmentShoppingListBinding? = null

    // This property is only valid between onCreateView and
    // onDestroyView.
    private val binding get() = _binding!!

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentShoppingListBinding.inflate(inflater, container, false)
        val root: View = binding.root

        thread {
            val items: Array<ShoppingListItem>
            try {
                items = getShoppingList(requireContext())
            } catch (e: ApiRequestException) {
                activity?.runOnUiThread {
                    Toast.makeText(context, e.message, Toast.LENGTH_LONG).show()
                }

                return@thread
            }

            activity?.runOnUiThread {
                binding.shoppingList.adapter = ShoppingItemAdapter(requireContext(), ArrayList(items.toMutableList()))
            }
        }

        binding.shoppingList.setOnItemClickListener { parent, view, position, id ->
            val item = parent.getItemAtPosition(position) as ShoppingListItem
            item.checked = !item.checked

            thread {
                try {
                    toggleShoppingItemChecked(requireContext(), item)
                } catch (e: ApiRequestException) {
                    Toast.makeText(requireContext(), e.message, Toast.LENGTH_LONG).show()
                }
            }

            val adapter = parent.adapter as ShoppingItemAdapter

            adapter.remove(item)
            adapter.insert(item, position)
        }

        binding.addToShoppingList.setOnClickListener {
            val view = requireActivity().layoutInflater.inflate(R.layout.dialog_add_to_shopping_list, null)

            val dialog = AlertDialog.Builder(activity)
                .setView(view)
                .setPositiveButton(R.string.add_label) { dialog, id ->
                    val dialog = dialog as AlertDialog

                    val amount = view.findViewById<EditText>(R.id.amount).text.toString().toDouble()
                    val unit = view.findViewById<Spinner>(R.id.unit_selector).selectedItem.toString().ifEmpty { null }
                    val name = view.findViewById<EditText>(R.id.name).text.toString()

                    dialog.getButton(AlertDialog.BUTTON_POSITIVE).isEnabled = false
                    dialog.getButton(AlertDialog.BUTTON_NEGATIVE).isEnabled = false

                    thread {
                        try {
                            addShoppingItem(requireContext(), name, amount, unit)
                        } catch (e: ApiRequestException) {
                            activity?.runOnUiThread {
                                Toast.makeText(context, e.message, Toast.LENGTH_LONG).show()
                            }
                        } finally {
                            activity?.runOnUiThread {
                                dialog.dismiss()
                            }
                        }
                    }
                }
                .setNegativeButton(R.string.cancel_label) { dialog, _ ->
                    dialog.cancel()
                }
                .create()

            dialog.show()

            val adapter = ArrayAdapter.createFromResource(requireContext(), R.array.units, android.R.layout.simple_spinner_item)
            adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)

            dialog.findViewById<Spinner>(R.id.unit_selector).adapter = adapter
        }

        return root
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }
}